// xmpp-client: A sample GTK client in Rust
// Copyright (C) 2024 Link Mauve <linkmauve@linkmauve.fr>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

use gtk::glib;

mod imp {
    use gtk::glib;
    use gtk::prelude::*;
    use gtk::subclass::prelude::*;
    use std::cell::RefCell;

    #[derive(glib::Properties)]
    #[properties(wrapper_type = super::Message)]
    pub struct Message {
        #[property(get, set)]
        date: RefCell<glib::DateTime>,
        #[property(get, construct_only)]
        author: RefCell<String>,
        #[property(get, set)]
        body: RefCell<String>,
    }

    impl Default for Message {
        fn default() -> Self {
            Message {
                date: RefCell::new(
                    glib::DateTime::from_iso8601("1970-01-01T00:00:00Z", None).unwrap(),
                ),
                author: RefCell::new(String::new()),
                body: RefCell::new(String::new()),
            }
        }
    }

    #[glib::object_subclass]
    impl ObjectSubclass for Message {
        const NAME: &'static str = "Message";
        type Type = super::Message;
        type ParentType = glib::Object;
    }

    #[glib::derived_properties]
    impl ObjectImpl for Message {}
}

glib::wrapper! {
    pub struct Message(ObjectSubclass<imp::Message>);
}

impl Message {
    pub fn new(date: &chrono::DateTime<chrono::Utc>, author: &str, body: &str) -> Self {
        let date = date.to_rfc3339();
        let date = glib::DateTime::from_iso8601(&date, Some(&glib::TimeZone::local())).unwrap();
        glib::Object::builder()
            .property("date", date)
            .property("author", author)
            .property("body", body)
            .build()
    }

    pub fn now(author: &str, body: &str) -> Self {
        let date = glib::DateTime::now(&glib::TimeZone::local()).unwrap();
        glib::Object::builder()
            .property("date", date)
            .property("author", author)
            .property("body", body)
            .build()
    }
}
